/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//========================================================================================================
// Object maps
//========================================================================================================
var EchoDateFieldMap = new Array();

//========================================================================================================
//
// The EchoDateField object
//
//========================================================================================================
function EchoDateField(htmlId, inputId, expansionState, hideHtmlId) {
	// members
	this.htmlId = htmlId;
	this.inputId = inputId;
	this.alignment = ALIGN_LEFT;
	this.expansionState = expansionState;
	this.hideHtmlId = null;
	if (hideHtmlId)
		this.hideHtmlId = hideHtmlId;

	EchoDateFieldMap[htmlId] = this;
}

//---------------------------------------------
// Called to hide/show select fields under the
// hideHtmlId
//---------------------------------------------
EchoDateField.prototype.toggleSelectFields = function(hideFlag) {
	if (browser.isIE) {
		if (hideFlag)
			epbe_hideSelectFieldsUnder(this.hideHtmlId);
		else
			epbe_showSelectFieldsUnder(this.hideHtmlId);
	}
}

//---------------------------------------------
// Called to position the dropDown the state of the DateField
//---------------------------------------------
EchoDateField.prototype.positionDropDown = function() {
	if (this.hideHtmlId != null) {
		var eOuter = getElementById(this.htmlId);
		var eDropDown = getElementById(this.hideHtmlId);

		var x = eOuter.getPageX();
		var y = eOuter.getPageY();
		var w = eOuter.getWidth();
		var h = eOuter.getHeight();
		var ddw = eDropDown.getWidth();
		var ddh = eDropDown.getHeight();
		var wAdjust = ddw - w;
		var scrollX = eOuter.calcScrollX();
		var scrollY = eOuter.calcScrollY();
		
		var newX = 0;
		var newY = 0;
		if (this.expansionState) {
			if (this.alignment == ALIGN_RIGHT) {
				newX  = x - wAjust - scrollX;
				newY  = y + h - scrollY;
			} else {
				newX  = x - scrollX;
				newY  = y + h - scrollY;
			}
			eDropDown.moveTo(newX,newY);
			eDropDown.setDisplayed(true);
			this.toggleSelectFields(true);
		} else {
			this.toggleSelectFields(false);
			eDropDown.setDisplayed(false);
		}
	}
}

//---------------------------------------------
// Called to toggle the state of the DateField
//---------------------------------------------
EchoDateField.prototype.toggle = function() {
	this.expansionState = !this.expansionState;
	this.positionDropDown();
	this.saveState();
}

//---------------------------------------------
// Called to save the state of the DateField
//---------------------------------------------
EchoDateField.prototype.saveState = function() {
	var stateStr = '|false';
	if (this.expansionState)
		stateStr = '|true';

	var elem = getElementById(this.inputId);
	if (elem != null)
		stateStr = elem.htmlE.value + stateStr;
	//
	// save our client side state
	if (E_setParameter)
		E_setParameter(this.htmlId, stateStr);
}

//---------------------------------------------
// Global function to toggle the date field
//---------------------------------------------
function g_datefield_toggle(htmlId) {
	var df = EchoDateFieldMap[htmlId];
	if (df != null)
		df.toggle();
}

//---------------------------------------------
// Global function to save the state of the datefield
//---------------------------------------------
function g_datefield_saveState(htmlId) {
	var df = EchoDateFieldMap[htmlId];
	if (df != null)
		df.saveState();
}

//---------------------------------------------
// Global function to hide select fields 
// at document onload
//---------------------------------------------
function g_datefield_onload(htmlId) {
	var df = EchoDateFieldMap[htmlId];
	if (df != null)
		df.positionDropDown();
}
