/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//========================================================================================================
// Object maps
//========================================================================================================
var EchoRecallTextFieldMap = new Array();


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Object to hold a value and rendering style class
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function RecallTextFieldItem(itemValue, itemClass) {
	this.itemValue = itemValue;
	this.itemClass = itemClass;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Our RecallTextField object.  2 stage construction
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function RecallTextField(htmlId, itemNormalClass, itemSelectedClass, actionOnEnter) {
	this.htmlId = htmlId;
	this.itemNormalClass = itemNormalClass;
	this.itemSelectedClass = itemSelectedClass;
	this.actionOnEnter = actionOnEnter;
	
	this.textFieldE = null;
	this.toggleE = null;
	this.listE = null;
	this.listContentE = null;
	this.options = new Array();
	this.matches = new Array();
	this.selectedIndex = -1;
	
	EchoRecallTextFieldMap[htmlId] = this;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Add an item to the list of possible options
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.addItem = function(itemValue, itemClass) {
	var newItemClass = itemClass;
	if (itemClass == undefined)
		newItemClass = this.itemNormalClass;
	var item = new RecallTextFieldItem(itemValue,newItemClass);
	this.options[this.options.length] = item;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Hides the popup list and clears the selection
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.hideList = function(itemValue) {
	this.listE.setVisible(false);
	this.selectedIndex = -1;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Gets the selected text or null
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.getSelectedItemText = function() {
	if (this.selectedIndex >= 0 && this.selectedIndex < this.matches.length) {
		var itemIndex = this.matches[this.selectedIndex];
		var item = this.options[itemIndex];
		if (item)
			return item.itemValue;
	}
	return null;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Gets the selected item class or null
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.getSelectedItemClass = function() {
	if (this.selectedIndex >= 0 && this.selectedIndex < this.matches.length) {
		var itemIndex = this.matches[this.selectedIndex];
		var item = this.options[itemIndex];
		if (item)
			return item.itemClass;
	}
	return null;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to set the focus to the textfield
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.setFocus = function() {
	this.textFieldE.htmlE.focus();
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called set the list content to the specified HTML
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.setListHTML = function(str) {
	var e = this.listContentE.htmlE;
	if (browser.isIE) {
		e.innerHTML = str;
	} else {
		var r = e.ownerDocument.createRange();
		r.selectNodeContents(e);
		r.deleteContents();
		var df = r.createContextualFragment(str);
		e.appendChild(df);
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called at key press time in the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epg_RecallTextFieldOnKey(event, htmlId) {
	var rctf = EchoRecallTextFieldMap[htmlId];
	if (rctf != null)
		rctf.onKey(event);
	else
		alert('ASSERT : RecallTextField error : object ' + htmlId + ' not found');
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called at key press time in the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.onKey  = function(event) {
	event = browser.isIE ? window.event : event;
	var key = browser.isIE ? event.keyCode : event.which;

	// TAB and SHIFT TAB
	if ((key == 9) || (key == 9 && event.shiftKey)) { 			
		return;
	}
	// DOWN ARROW
	else if (key == 40) { 		
		epbe_CancelEvent(event);
		if (! this.listE.isVisible()) {
			this.rebuildMatches();
			this.selectedIndex = 0;
		} else {
			this.selectedIndex++;
			if (this.selectedIndex >= this.matches.length)
				this.selectedIndex = -1;
		}		
	// UP ARROW
	} else if (key == 38) { 	
		epbe_CancelEvent(event);
		if (this.selectedIndex == -1)
			this.selectedIndex = this.matches.length-1;
		else {
			this.selectedIndex--;
		}
	// ENTER and SHIFT ENTER KEY
	} else if (key == 13 || (key == 13 && event.shiftKey)) {		
		epbe_CancelEvent(event);
		if (this.listE.isVisible()) {
			var selectedText = this.getSelectedItemText();
			if (selectedText) {
				this.setText(selectedText);
				this.hideList();
			}
		} else {
			// save the state of things!
			if (this.actionOnEnter) {
				this.saveState();
				E_setAction(this.htmlId, "enter");
			}
		}
		return;
	// ANY THING ELSE
	} else {
		//
		// we only rebuild our match list if the text changes
		this.rebuildMatches();
		this.selectedIndex = -1;
	}
	var itemValue = this.rebuildListItems();
	if (itemValue)
		this.setText(itemValue);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Sets the  array of "indexes" of items that match a 
// the current text field value according to the rules
//	  * if len <= 0 then show entire list
//	  * if len > 0 and no matches then no list
//	  * if len > 0 and matches then show list of just matches
//	  * if showAll == true then show entire list
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.rebuildMatches = function(showAll) {
	var textVal = this.textFieldE.htmlE.value;
	var matches = null;
	if (textVal.length == 0 || showAll) {
		matches = new Array(this.options.length);
		for (i = 0; i < this.options.length; i++) {
			matches[i] = i;
		}
	} else {
		matches = new Array();
		for (i = 0; i < this.options.length; i++) {
			var val = this.options[i].itemValue;
			if (val.indexOf(textVal) == 0) {
				matches[matches.length] = i;
			}
		}
	}
	this.matches = matches;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to rebuild the list items based on matching rules
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.rebuildListItems = function() {
	var returnValue = null;
	if (this.matches.length <=0) {
		this.hideList();
	} else {
		var s = '';
		s += '<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">';
		for (i = 0; i < this.matches.length; i++) {
			var matchIndex = this.matches[i];
			var id = this.htmlId + i;
			
			var itemValue = this.options[matchIndex].itemValue;
			var itemClass = this.options[matchIndex].itemClass;
			if (itemClass == null)
				itemClass = this.itemNormalClass;
			//
			// if the currently item is the selected item, then
			// bloody well select it
			var currentItemClass = itemClass;
			if (i == this.selectedIndex) {
				currentItemClass = this.itemSelectedClass;
				returnValue = itemValue;
			}
			
			
			s += '<tr><td nowrap';
			s += ' id="'+id+'"';
			s += ' class="' + currentItemClass + '" ';
			s += ' onmouseover="epg_RecallTextFieldMouseOver(\'' + this.htmlId + '\',' + i + ',\'' + itemClass + '\',\'' + this.itemSelectedClass + '\');"';
			s += ' onmouseout="this.className=\'' + itemClass + '\';"';
			s += ' onmousedown="epg_RecallTextFieldItemClick(event,\'' + this.htmlId + '\',\'' + itemValue + '\');"';
			s += '>&nbsp;';
			s += itemValue;
			s += '&nbsp;';
			s += '</td></tr>';
		}
		s += '</table>';
		
		this.setListHTML(s);
		//
		// make sure the list is at least as wide as the text field
		var wT = this.textFieldE.getWidth();
		var wL = this.listE.getWidth();
		if (wT > wL)
			this.listE.setWidth(wT);
		this.listE.setVisible(true);
	}
	return returnValue;
}	

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called when the mouse goes over a list item
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epg_RecallTextFieldMouseOver(htmlId, index) {
	var rctf = EchoRecallTextFieldMap[htmlId];
	if (rctf != null) {
		var tdE = getElementById(htmlId + index);
		var oldE = getElementById(htmlId + rctf.selectedIndex);
		if (oldE != null) {
			var itemClass = rctf.getSelectedItemClass();
			if (itemClass)
				oldE.htmlE.className = itemClass;
		}
		rctf.selectedIndex = index;
		tdE.htmlE.className = rctf.itemSelectedClass;
	} else {
		alert('ASSERT : RecallTextField error : object ' + htmlId + ' not found');
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to toggle the display of the list
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.toggle = function() {
	if (this.listE.isVisible()) {
		this.hideList();
	} else {
		this.rebuildMatches(true);
		this.rebuildListItems();
	}
	this.setFocus();
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called at key press time in the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epg_RecallTextFieldToggle(event, htmlId) {
	var rctf = EchoRecallTextFieldMap[htmlId];
	if (rctf != null)
		rctf.toggle(event);
	else
		alert('ASSERT : RecallTextField error : object ' + htmlId + ' not found');
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to set the text of the textfield
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.setText = function(textValue) {
		var delayedFunc = 'epg_RecallTextFieldSetText("' + this.htmlId + '","'+ textValue + '")';
		window.setTimeout(delayedFunc,10);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called at set the text of the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epg_RecallTextFieldSetText(htmlId, textValue) {
	var rctf = EchoRecallTextFieldMap[htmlId];
	if (rctf != null) {
		rctf.textFieldE.htmlE.value = textValue;
		rctf.setFocus();
	} else {
		alert('ASSERT : RecallTextField error : object ' + htmlId + ' not found');
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called at item click time
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epg_RecallTextFieldItemClick(event, htmlId, textValue) {
	var rctf = EchoRecallTextFieldMap[htmlId];
	if (rctf != null) {
		rctf.hideList();
		rctf.setText(textValue);
	} else {
		alert('ASSERT : RecallTextField error : object ' + htmlId + ' not found');
	}
	return false;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called at load time to init the RecallTextField
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.init = function(itemValue) {
	this.textFieldE = getElementById(this.htmlId);
	if (this.textFieldE == null)
		alert('ASSERT : RecallTextField init error : textfield not present');
		
	this.listE = getElementById(this.htmlId+"List");
	if (this.listE == null)
		alert('ASSERT : RecallTextField init error : list not present');

	this.listContentE = getElementById(this.htmlId+"ListContent");
	if (this.listContentE == null)
		alert('ASSERT : RecallTextField init error : list content not present');

	this.toggleE = getElementById(this.htmlId+"Toggle");
	if (this.listContentE == null)
		alert('ASSERT : RecallTextField init error : toggle not present');
		
	// position the elements next to the textfield
	var x = this.textFieldE.getPageX();
	var y = this.textFieldE.getPageY() + this.textFieldE.getHeight();
	this.listE.setX(x);
	this.listE.setY(y-2);
	
	// set our default match list
	this.rebuildMatches();
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called at load time to init the RecallTextField
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epg_RecallTextFieldOnLoad(htmlId) {
	var rctf = EchoRecallTextFieldMap[htmlId];
	if (rctf != null)
		rctf.init();
	else
		alert('ASSERT : RecallTextField error : object ' + htmlId + ' not found');
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to save the state of the component
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
RecallTextField.prototype.saveState = function() {
	E_setParameter(this.htmlId, this.textFieldE.htmlE.value);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called at set the text of the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epg_RecallTextFieldSaveState(htmlId) {
	var rctf = EchoRecallTextFieldMap[htmlId];
	if (rctf != null)
		rctf.saveState();
	else
		alert('ASSERT : RecallTextField error : object ' + htmlId + ' not found');
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called at document onclick
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epRecallTextFieldDocumentOnClickHandler(event) {
	var targetHtmlE = null;
	if (browser.isIE)
		targetHtmlE = window.event.srcElement;
	else
		targetHtmlE = event.target;

	for (x in EchoRecallTextFieldMap) {
		var rctf = EchoRecallTextFieldMap[x];
		if (rctf.toggleE.htmlE != targetHtmlE)
			rctf.hideList();
	}
}

epbe_registerDocumentEventHandler("click",epRecallTextFieldDocumentOnClickHandler,"recalltextfield");
